/*
    v0.01 - original
*/
#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "eit.h"


#define ID_USERTAP 0xbdb01e01
TAP_ID( ID_USERTAP );

TAP_PROGRAM_NAME("crid v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("a tap to extract Series Link information");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

extern void TSRCommanderConfigDialog();
extern bool TSRCommanderExitTAP();
extern int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//
extern type_menu main_menu;

extern int TAP_Main(void);
extern dword TAP_EventHandler(word event, dword param1, dword param2);
extern int main_menu_handler(type_menu *menu, dword param1);
extern int key_handler(dword param1);

extern int init(void);
extern int close(void);

extern void init_crc(void);
extern dword crc8(dword crc, byte *data, int len);

//----------------------------------------------------------------------------

int eit_enabled = 0;
int eit_hook_id = -2;
int apply_eit_hook();
int remove_eit_hook();
dword eit_hook(dword param1, dword param2);

extern int genre_enabled;
extern int genre_hook_id;
extern int apply_genre_hook();
extern int remove_genre_hook();
extern dword genre_hook(dword param1, dword param2);

dword service_id_filter(byte *buf);

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
bool TSRCommanderExitTAP()
{
    close();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
    TAP_ExitNormal();
    ShowMenu(&main_menu);
}

int is_tsr_commander_running(void)
{
    int i;
	type_TAPProcess *TAPProcess;

    if (fw_tap_table <= 0) return 0;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
                if (TAPProcess->header->id == 0x81001021) {  //tap_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//main
//
char *tap_name;
type_menu main_menu = {
    0,
    0,
    {0},
    0,
    (char *)__tap_program_name__,
    {
        {1, "xxx",                 },
        {0, "Exit Tap",            },
    },
};

//----------------------------------------------------------------------------
int TAP_Main(void)
{
    int ret;

    tap_name =  (char *)__tap_program_name__;

    sprintf(strLog, "Opening %s", tap_name);
    log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);

    ret = exTAP_Init();
    if (ret < 0) {
        sprintf(strLog, "Unsupported platform - %d:%04x", fw_sysid, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
        return 0;
    }

    init();
    TSRCommanderInit(0, TRUE);

    return 1;                 //0=application mode, 1=tsr mode
}

int init(void)
{

    eit_enabled = 0;
    genre_enabled = 0;
    main_menu.visible = 0;
    init_crc();

    apply_eit_hook();
    apply_genre_hook();

    return 0;
}

int close(void)
{
    sprintf(strLog, "Closing  %s", tap_name);
    log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);

    remove_eit_hook();
    remove_genre_hook();

    DeleteMenu(&main_menu);
    if (message_rgn) TAP_Osd_Delete(message_rgn);
    if (OpLog) TAP_Hdd_Fclose(OpLog);
    TSRCommanderErase();

    TAP_Print("\nClosing %s\n",__tap_program_name__);
    TAP_Exit();
    return 0;
}

//----------------------------------------------------------------------------
dword TAP_EventHandler(word event, dword param1, dword param2)
{
    TYPE_TapChInfo channel_info;
    int channel_type, channel_number;
    int ret;
    dword state, subState;

    switch (event) {
        case EVT_IDLE:
//            TAP_Print("\nEVT_IDLE, event=%x, param1=%x, param2=%x", event, param1, param2);
            if (message_showing) {
                if (TAP_GetTick() > message_showing) {
                    TAP_Osd_Delete(message_rgn);    // release rgn-handle
                    message_showing = 0;
                    message_rgn = 0;
                }
            }
    		TSRCommanderWork();
            break;

        case EVT_KEY:
//            TAP_Print("\nEVT_KEY, event=%x, param1=%x, param2=%x", event, param1, param2);
                TAP_GetState(&state, &subState);
                if (main_menu.visible) {
                    param1 = main_menu_handler(&main_menu, param1);
                }
                else
                if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                    param1 = key_handler(param1);
                }
            break;
        case EVT_HOOK:
            if (param1 == eit_hook_id) param1 = eit_hook(param1, param2);
            if (param1 == genre_hook_id) param1 = genre_hook(param1, param2);
            break;
        default:
//            TAP_Print("\nEVT_???, event=%x, param1=%x, param2=%x", event, param1, param2);
            break;
    }
    return param1;
}

int main_menu_handler(type_menu *menu, dword param1)
{
    int selection;
    int i;
    int num_events;
    int ret;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            TAP_EnterNormal();
            switch (selection) {
                case 0:
                    close();
                    return 0;
            }
            break;
       case RKEY_Recall:
            DeleteMenu(menu);
            TAP_EnterNormal();
            return param1;
        case RKEY_Exit:
            DeleteMenu(menu);
            TAP_EnterNormal();
            return 0;
        default:
            TAP_Win_Action(&menu->win, param1);
            return 0;
    }
    return param1;
}


int key_handler(dword param1)
{
    switch(param1) {
       //only if tsrcommander is not running
        case RKEY_Recall:
            if (!is_tsr_commander_running()) {
                TAP_ExitNormal();
                ShowMenu(&main_menu);
                return 0;
            }
            break;
    }
    return param1;
}

//----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//eit stuff
//
//hook the process_eit() function
//this code just filters by service_id
//allow through all now/next (table_id=0x4e)
//only allow through enabled schedule (table_id=50/51)
//could add max# days per channels
//should never allow if servicve_id does not exists - e.g when 4tv data
//
type_hook eit_hook_patch[] = {
// sysid, firmware, enable, address,    length, original_data
    {456, 0x1209,   1,      0x8014ed60, 8,      {0x27, 0xbd, 0xff, 0x28,  0xaf, 0xb0, 0x00, 0xb8, }, },  //process_eit
    {456, 0x1225,   1,      0x8014fc2c, 8,      {0x27, 0xbd, 0xff, 0x28,  0xaf, 0xb0, 0x00, 0xb8, }, },  //process_eit
    {456, 0x1288,   1,      0x80155194, 8,      {0x27, 0xbd, 0xff, 0x18,  0xaf, 0xb0, 0x00, 0xc8, }, },  //process_eit
};

byte *to;

int apply_eit_hook(void)
{
    int i;

    to = (byte *)malloc(0x1000);
    if (to == 0) {
        sprintf(strLog, "ERROR: apply_eit_hook(), malloc failed, to = %x, len=%x", to, 0x1000);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
        return -1;
    }

    eit_hook_id = exTAP_ApplyHook(&eit_hook_patch[0]);
    if (eit_hook_id < 0) {
        sprintf(strLog, "ERROR: apply_eit_hook(), firmware version %04x not supported", _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
        return -1;
    }

    eit_enabled = 1;
    return 0;
}

int remove_eit_hook(void)
{
    int ret;

    if (eit_hook_id >=0) {
        ret = exTAP_RemoveHook(eit_hook_id);
        if (ret == 0) {
            eit_hook_id = -1;
        }
    }
    free(to);

    eit_enabled = 0;
    return 0;
}

dword eit_hook(dword param1, dword param2)
{
    type_hook_registers *reg;

//TAP_Print("\nEVT_HOOK_eit, param1=%x, param2=%x", param1, param2);
    reg = (type_hook_registers *)param2;
    if ((reg->a0 > 0x80000000) && (reg->a0 < 0x84000000)) {
        reg->a0 = service_id_filter((byte *)(reg->a0));
    }
    return param1;
}



dword service_id_filter(byte *from)
{
    dword crc;
    byte table_id;
    word section_length;
    word table_id_extension;
    byte version_number;
    byte current_next_indicator;
    byte section_number;
    byte last_section_number;
    word transport_stream_id;
    word original_network_id;

    int i;
    int to_len;

    //do crc check
    section_length         = 3 + (((from[1]<<8) + from[2]) & 0xfff);
    crc = crc32(from, section_length);
    if (crc) {
//        TAP_Print("\nWARNING: *** CRC ERROR ***, section_length=%x", section_length);
//        dump((dword)&from[0], 0x40);
        return (dword)from;
    }

    table_id               = from[0];
    section_length         = 3 + (((from[1]<<8) + from[2]) & 0xfff);
    table_id_extension     = ((from[3]<<8) + from[4]) & 0xffff;
    version_number         = (from[5]>>1) & 0x1f;
    current_next_indicator = (from[5]>>0) & 0x1;
    section_number         = from[6];
    last_section_number    = from[7];
    transport_stream_id    = ((from[8]<<8) + from[9]) & 0xffff;         //eit only
    original_network_id    = ((from[10]<<8) + from[11]) & 0xffff;       //eit only

//    TAP_Print("\nEIT:  0012:%02x:%04x %3d/%3d: %08x:%x", table_id, table_id_extension, section_number, last_section_number, buf, section_length);

    //reject unless eit
    switch (table_id) {
        case 0x4e:
        case 0x4f:
            //allow all now_next
            break;
        case 0x50:
        case 0x51:
        case 0x60:
        case 0x61:
            break;
        default:
            sprintf(strLog,"WARNING: service_id_filter(), invalid table_id = %02x", from[0]);
            log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
            return (dword)from;
            break;
    }

    //now process eit section
    convert_event_information_section(from, to, &to_len);

//    if (to_len > 0x1000) {
//        TAP_Print("\nERROR: service_id_filter(), to_len=%x",to_len);
////        dump((dword)&to[0], to_len);
//        dump((dword)&to[0], 0x40);
//    }

    return (dword)&to[0];
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//*
//* The FCS-32 generator polynomial: x**0 + x**1 + x**2 + x**4 + x**5
//*                      + x**7 + x**8 + x**10 + x**11 + x**12 + x**16
//*                      + x**22 + x**23 + x**26 + x**32.
//*
#define P32             0xedb88320  // the polymonial
#define CRC_POLYNOMIAL  0x04c11db7
//*
//* FCS lookup table as calculated by the table generator.
//*
static dword fcstab8[0x100];

static dword bitrevN(int n, dword data)
{
    int i;
    dword result;

    result = 0;
    for (i=0; i<n; i++) {
        result += ((data >> (n-1-i)) & 1) << (i);
    }
    return result;
}

void init_crc(void)
{
    int b;
    int b2;
    dword v;
    int i;

    for (b=0; b<(1<<8); b++) {
        b2 = bitrevN(8,b);
        v = (dword)b;
        for(i=0; i<8; i++) {
            v = (v >> 1) ^ ((v & 1) ? P32 : 0);
        }

        fcstab8[b2] = bitrevN(32,v);
    }
}

// 32 bit crc, 8 bit data, 8 bits at a time
// lookup table = 2^8=256 words
dword crc8(dword crc, byte *data, int len)
{
    int i;
    int index;
    dword dataout32;

    for(i=0; i<len; i++) {
        index = ((crc>>24) ^ (data[i])) & 0xff;
        dataout32 = fcstab8[index];
        crc = (crc << 8) ^ dataout32;
    }

    return crc;
}
