//genre.c
//
//
#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "eit.h"

extern dword get_genre_str(char *str, type_eventtable *evt);
//-----------------------------------------------------------------------------

int genre_enabled = 0;
int genre_hook_id = -2;


//----------------------------------------------------------------------------
//
//hook the eit() function
//
dword fw_get_genre_str = 0x8012b19c;

type_hook genre_hook_patch[] = {
// sysid, firmware, enable, address,    length, original_data
    {456, 0x1288,   1,      0x8012b19c, 8,      {0x27, 0xbd, 0xff, 0xe8,  0xaf, 0xbf, 0x00, 0x14, }, },  //fw_get_genre_str
    {0},
};

int apply_genre_hook(void)
{

    if (fw_get_genre_str == 0) {
        genre_enabled = 0;
        TAP_Print("\nERROR: apply_genre_hook(), firmware version %04x not supported", _appl_version);
        return -1;
    }
    //fix the address;
    genre_hook_patch[0].sysid = fw_sysid;
    genre_hook_patch[0].revision = _appl_version;
    genre_hook_patch[0].enable = 1;
    genre_hook_patch[0].addr = (dword)fw_get_genre_str;

    genre_hook_id = exTAP_ApplyHook(&genre_hook_patch[0]);
    if (genre_hook_id < 0) {
        genre_enabled = 0;
        TAP_Print("\nERROR: apply_genre_hook(), firmware version %04x not supported", _appl_version);
        return -1;
    }
    genre_enabled = 1;
    return 0;
}

int remove_genre_hook(void)
{
    int ret;

    if (genre_hook_id >=0) {
        ret = exTAP_RemoveHook(genre_hook_id);
        if (ret == 0) genre_hook_id = -1;
    }
    genre_enabled = 0;
    return 0;
}


dword genre_hook(dword param1, dword param2)
{
    type_hook_registers *reg;

    reg = (type_hook_registers *)param2;
//    dump_reg(reg);
    reg->a0 = get_genre_str((char *)reg->a0, (type_eventtable *)reg->a1);

    return param1;
//    return -1;      //override firmware function
}

dword get_genre_str(char *str, type_eventtable *evt)
{
    char event_name[0x100];
    byte num_extended;
    word extended_length;
    byte *extended_event_name;
    int i;
    int offset, ooffset;

    //use the event table, pass back the seres link status ...
    memcpy(event_name, evt->event_name, evt->event_name_length);
    event_name[evt->event_name_length] = '\0';

//!!!the order of the internal type_eventtable structure is different for v12.88!!!
    if ((fw_sysid==456)&&(_appl_version>=0x1288)) {
        num_extended = *(byte *)(((dword)&evt->event_name)+9);
        extended_length = *(word *)(((dword)&evt->event_name)+10);
        extended_event_name = (byte *)*(dword *)(((dword)&evt->event_name)+4);
    }
    else {
        num_extended = evt->num_extended;
        extended_length = evt->extended_length;
        extended_event_name = evt->extended_event_name;
    }

    offset = 0;
    for (i=0; i<num_extended; i++) {
        ooffset = offset;
        offset += strlen(&extended_event_name[offset]);
        offset ++;

//        if (extended_event_name[offset]) {
//            TAP_Print(", %s", &extended_event_name[offset]);
//        }

        //check for "crid=32"
        if (strncmp(&extended_event_name[ooffset], "crid=32", 7) == 0) {
            if (extended_event_name[offset]) {
                sprintf(str, "SeriesLink-");
                return (dword)&str[11];
            }
        }

        offset += strlen(&extended_event_name[offset]);
        offset ++;
    }

    return (dword)str;
}
